@@ -10,7 +10,6 @@
 #define BOTAN_BIGINT_H_
 
 #include <botan/exceptn.h>
-#include <botan/mem_ops.h>
 #include <botan/secmem.h>
 #include <botan/types.h>
 #include <iosfwd>
@@ -28,7 +27,11 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
       /**
        * Base enumerator for encoding and decoding
        */
-      enum Base { Decimal = 10, Hexadecimal = 16, Binary = 256 };
+      enum Base {
+         Decimal BOTAN_DEPRECATED("All functions using this enum are deprecated") = 10,
+         Hexadecimal BOTAN_DEPRECATED("All functions using this enum are deprecated") = 16,
+         Binary BOTAN_DEPRECATED("All functions using this enum are deprecated") = 256
+      };
 
       /**
        * Sign symbol definitions for positive and negative numbers
@@ -48,7 +51,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
       /**
        * Create a 1-value BigInt
        */
-      static BigInt one() { return BigInt::from_word(1); }
+      static BigInt one() { return BigInt::from_u64(1); }
 
       /**
        * Create BigInt from an unsigned 64 bit integer
@@ -60,17 +63,20 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * Create BigInt from a word (limb)
        * @param n initial value of this BigInt
        */
+      //BOTAN_DEPRECATED("Use BigInt::from_u64 instead")
       static BigInt from_word(word n);
 
       /**
        * Create BigInt from a signed 32 bit integer
        * @param n initial value of this BigInt
        */
-      static BigInt from_s32(int32_t n);
+      BOTAN_DEPRECATED("Use BigInt::from_u64 plus negation if required instead") static BigInt from_s32(int32_t n);
 
       /**
        * Create BigInt from an unsigned 64 bit integer
        * @param n initial value of this BigInt
+       *
+       * Prefer BigInt::from_u64
        */
       BigInt(uint64_t n);
 
@@ -87,20 +93,34 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        *
        * @param str the string to parse for an integer value
        */
+      //BOTAN_DEPRECATED("Use BigInt::from_string")
       explicit BigInt(std::string_view str);
 
+      /**
+       * Create BigInt from a string.
+       *
+       * If the string starts with 0x the rest of the string will be
+       * interpreted as hexadecimal digits. Otherwise, it will be
+       * interpreted as a decimal number.
+       *
+       * A prefix of "-" will result in a negative integer
+       *
+       * @param str the string to parse for an integer value
+       */
+      static BigInt from_string(std::string_view str);
+
       /**
        * Create a BigInt from an integer in a byte array
        * @param buf the byte array holding the value
        * @param length size of buf
        */
-      BigInt(const uint8_t buf[], size_t length);
+      BigInt(const uint8_t buf[], size_t length) { assign_from_bytes(std::span{buf, length}); }
 
       /**
        * Create a BigInt from an integer in a byte array
-       * @param vec the byte vector holding the value
+       * @param bytes the byte vector holding the value
        */
-      explicit BigInt(std::span<const uint8_t> vec) : BigInt(vec.data(), vec.size()) {}
+      explicit BigInt(std::span<const uint8_t> bytes) { assign_from_bytes(bytes); }
 
       /**
        * Create a BigInt from an integer in a byte array
@@ -108,7 +128,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param length size of buf
        * @param base is the number base of the integer in buf
        */
-      BigInt(const uint8_t buf[], size_t length, Base base);
+      BOTAN_DEPRECATED("For hex/decimal use from_string") BigInt(const uint8_t buf[], size_t length, Base base);
 
       /**
        * Create a BigInt from an integer in a byte array
@@ -121,6 +141,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param max_bits if the resulting integer is more than max_bits,
        *        it will be shifted so it is at most max_bits in length.
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       static BigInt from_bytes_with_max_bits(const uint8_t buf[], size_t length, size_t max_bits);
 
       /**
@@ -138,14 +159,14 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * Create BigInt of specified size, all zeros
        * @param n size of the internal register in words
        */
-      static BigInt with_capacity(size_t n);
+      BOTAN_DEPRECATED("Deprecated no replacement") static BigInt with_capacity(size_t n);
 
       /**
        * Move constructor
        */
       BigInt(BigInt&& other) { this->swap(other); }
 
-      ~BigInt() { const_time_unpoison(); }
+      ~BigInt() { _const_time_unpoison(); }
 
       /**
        * Move assignment
@@ -174,7 +195,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
 
       friend void swap(BigInt& x, BigInt& y) { x.swap(y); }
 
-      void swap_reg(secure_vector<word>& reg) {
+      BOTAN_DEPRECATED("Deprecated no replacement") void swap_reg(secure_vector<word>& reg) {
          m_data.swap(reg);
          // sign left unchanged
       }
@@ -183,7 +204,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * += operator
        * @param y the BigInt to add to this
        */
-      BigInt& operator+=(const BigInt& y) { return add(y.data(), y.sig_words(), y.sign()); }
+      BigInt& operator+=(const BigInt& y) { return add(y._data(), y.sig_words(), y.sign()); }
 
       /**
        * += operator
@@ -195,7 +216,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * -= operator
        * @param y the BigInt to subtract from this
        */
-      BigInt& operator-=(const BigInt& y) { return sub(y.data(), y.sig_words(), y.sign()); }
+      BigInt& operator-=(const BigInt& y) { return sub(y._data(), y.sig_words(), y.sign()); }
 
       /**
        * -= operator
@@ -285,10 +306,13 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        */
       bool operator!() const { return (!is_nonzero()); }
 
+      //BOTAN_DEPRECATED("Just use operator+/operator-")
       static BigInt add2(const BigInt& x, const word y[], size_t y_words, Sign y_sign);
 
+      //BOTAN_DEPRECATED("Just use operator+/operator-")
       BigInt& add(const word y[], size_t y_words, Sign sign);
 
+      //BOTAN_DEPRECATED("Just use operator+/operator-")
       BigInt& sub(const word y[], size_t y_words, Sign sign) {
          return add(y, y_words, sign == Positive ? Negative : Positive);
       }
@@ -298,13 +322,13 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param y the BigInt to multiply with this
        * @param ws a temp workspace
        */
-      BigInt& mul(const BigInt& y, secure_vector<word>& ws);
+      BOTAN_DEPRECATED("Just use operator*") BigInt& mul(const BigInt& y, secure_vector<word>& ws);
 
       /**
        * Square value of *this
        * @param ws a temp workspace
        */
-      BigInt& square(secure_vector<word>& ws);
+      BOTAN_DEPRECATED("Deprecated no replacement") BigInt& square(secure_vector<word>& ws);
 
       /**
        * Set *this to y - *this
@@ -312,6 +336,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param y_words length of y in words
        * @param ws a temp workspace
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       BigInt& rev_sub(const word y[], size_t y_words, secure_vector<word>& ws);
 
       /**
@@ -321,6 +346,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param mod the positive modulus
        * @param ws a temp workspace
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       BigInt& mod_add(const BigInt& y, const BigInt& mod, secure_vector<word>& ws);
 
       /**
@@ -330,6 +356,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param mod the positive modulus
        * @param ws a temp workspace
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       BigInt& mod_sub(const BigInt& y, const BigInt& mod, secure_vector<word>& ws);
 
       /**
@@ -340,6 +367,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param mod the positive modulus
        * @param ws a temp workspace
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       BigInt& mod_mul(uint8_t y, const BigInt& mod, secure_vector<word>& ws);
 
       /**
@@ -349,7 +377,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * mod and performs repeated subtractions. It should not be used if
        * *this is much larger than mod, instead use modulo operator.
        */
-      size_t reduce_below(const BigInt& mod, secure_vector<word>& ws);
+      BOTAN_DEPRECATED("Deprecated no replacement") size_t reduce_below(const BigInt& mod, secure_vector<word>& ws);
 
       /**
        * Return *this % mod
@@ -361,6 +389,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * Performs exactly bound subtractions, so if *this is >= bound*mod then the
        * result will not be fully reduced. If bound is zero, nothing happens.
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       void ct_reduce_below(const BigInt& mod, secure_vector<word>& ws, size_t bound);
 
       /**
@@ -442,8 +471,8 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param set_it if the bit should be set
        */
       void conditionally_set_bit(size_t n, bool set_it) {
-         const size_t which = n / BOTAN_MP_WORD_BITS;
-         const word mask = static_cast<word>(set_it) << (n % BOTAN_MP_WORD_BITS);
+         const size_t which = n / (sizeof(word) * 8);
+         const word mask = static_cast<word>(set_it) << (n % (sizeof(word) * 8));
          m_data.set_word_at(which, word_at(which) | mask);
       }
 
@@ -457,14 +486,14 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * Clear all but the lowest n bits
        * @param n amount of bits to keep
        */
-      void mask_bits(size_t n) { m_data.mask_bits(n); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void mask_bits(size_t n) { m_data.mask_bits(n); }
 
       /**
        * Return bit value at specified position
        * @param n the bit offset to test
        * @result true, if the bit at position n is set, false otherwise
        */
-      bool get_bit(size_t n) const { return ((word_at(n / BOTAN_MP_WORD_BITS) >> (n % BOTAN_MP_WORD_BITS)) & 1); }
+      bool get_bit(size_t n) const { return ((word_at(n / (sizeof(word) * 8)) >> (n % (sizeof(word) * 8))) & 1); }
 
       /**
        * Return (a maximum of) 32 bits of the complete value
@@ -473,14 +502,14 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @result the integer extracted from the register starting at
        * offset with specified length
        */
-      uint32_t get_substring(size_t offset, size_t length) const;
+      BOTAN_DEPRECATED("Deprecated no replacement") uint32_t get_substring(size_t offset, size_t length) const;
 
       /**
        * Convert this value into a uint32_t, if it is in the range
        * [0 ... 2**32-1], or otherwise throw an exception.
        * @result the value as a uint32_t if conversion is possible
        */
-      uint32_t to_u32bit() const;
+      BOTAN_DEPRECATED("Deprecated no replacement") uint32_t to_u32bit() const;
 
       /**
        * Convert this value to a decimal string.
@@ -517,9 +546,11 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        */
       word word_at(size_t n) const { return m_data.get_word_at(n); }
 
-      void set_word_at(size_t i, word w) { m_data.set_word_at(i, w); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void set_word_at(size_t i, word w) { m_data.set_word_at(i, w); }
 
-      void set_words(const word w[], size_t len) { m_data.set_words(w, len); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void set_words(const word w[], size_t len) {
+         m_data.set_words(w, len);
+      }
 
       /**
        * Tests if the sign of the integer is negative
@@ -552,7 +583,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
       /**
        * Flip the sign of this BigInt
        */
-      void flip_sign() { set_sign(reverse_sign()); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void flip_sign() { set_sign(reverse_sign()); }
 
       /**
        * Set sign of the integer
@@ -597,40 +628,44 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
 
       /**
        * Get the number of high bits unset in the top (allocated) word
-       * of this integer. Returns BOTAN_MP_WORD_BITS only iff *this is
+       * of this integer. Returns (sizeof(word) * 8) only iff *this is
        * zero. Ignores sign.
        */
-      size_t top_bits_free() const;
+      BOTAN_DEPRECATED("Deprecated no replacement") size_t top_bits_free() const;
 
       /**
        * Return a mutable pointer to the register
        * @result a pointer to the start of the internal register
        */
-      word* mutable_data() { return m_data.mutable_data(); }
+      BOTAN_DEPRECATED("Deprecated no replacement") word* mutable_data() { return m_data.mutable_data(); }
 
       /**
        * Return a const pointer to the register
        * @result a pointer to the start of the internal register
        */
-      const word* data() const { return m_data.const_data(); }
+      BOTAN_DEPRECATED("Deprecated no replacement") const word* data() const { return m_data.const_data(); }
 
       /**
        * Don't use this function in application code
        */
-      secure_vector<word>& get_word_vector() { return m_data.mutable_vector(); }
+      BOTAN_DEPRECATED("Deprecated no replacement") secure_vector<word>& get_word_vector() {
+         return m_data.mutable_vector();
+      }
 
       /**
        * Don't use this function in application code
        */
-      const secure_vector<word>& get_word_vector() const { return m_data.const_vector(); }
+      BOTAN_DEPRECATED("Deprecated no replacement") const secure_vector<word>& get_word_vector() const {
+         return m_data.const_vector();
+      }
 
       /**
        * Increase internal register buffer to at least n words
        * @param n new size of register
        */
-      void grow_to(size_t n) const { m_data.grow_to(n); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void grow_to(size_t n) const { m_data.grow_to(n); }
 
-      void resize(size_t s) { m_data.resize(s); }
+      BOTAN_DEPRECATED("Deprecated no replacement") void resize(size_t s) { m_data.resize(s); }
 
       /**
        * Fill BigInt with a random number with size of bitsize
@@ -645,11 +680,58 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        */
       void randomize(RandomNumberGenerator& rng, size_t bitsize, bool set_high_bit = true);
 
+      /**
+       * Serialize the absolute value of this BigInt as a big endian
+       * encoding.
+       *
+       * If out is smaller than the total bytes of the BigInt then
+       * an exception is thrown.
+       *
+       * If out is larger than the total bytes of the BigInt then the
+       * necessary number of zeros are prefixed to produce the desired
+       * output length
+       *
+       * Zero-padding the binary encoding is useful to ensure that other
+       * applications correctly parse the encoded value as "positive integer",
+       * as a leading 1-bit may be interpreted as a sign bit. It also is
+       * necessary when using a fixed size encoding for the integers.
+       *
+       * @param out destination byte span for the integer value
+       */
+      void serialize_to(std::span<uint8_t> out) const;
+
+      /**
+       * Serialize the value of this BigInt as a big endian encoding,
+       * always returning the specified number of bytes.
+       *
+       * Throws if the BigInt is too large to encode in the length
+       * specified.
+       */
+      template <typename T = std::vector<uint8_t>>
+      T serialize(size_t len) const {
+         // TODO this supports std::vector and secure_vector
+         // it would be nice if this also could work with std::array as in
+         //   bn.serialize_to<std::array<uint8_t, 32>>(32);
+         T out(len);
+         this->serialize_to(out);
+         return out;
+      }
+
+      /**
+       * Serialize the value of this BigInt as a big endian encoding.
+       */
+      template <typename T = std::vector<uint8_t>>
+      T serialize() const {
+         return serialize<T>(this->bytes());
+      }
+
       /**
        * Store BigInt-value in a given byte array
        * @param buf destination byte array for the integer value
        */
-      void binary_encode(uint8_t buf[]) const;
+      BOTAN_DEPRECATED("Use BigInt::serialize_to") void binary_encode(uint8_t buf[]) const {
+         this->serialize_to(std::span{buf, this->bytes()});
+      }
 
       /**
        * Store BigInt-value in a given byte array. If len is less than
@@ -665,63 +747,62 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param buf destination byte array for the integer value
        * @param len how many bytes to write
        */
-      void binary_encode(uint8_t buf[], size_t len) const;
+      BOTAN_DEPRECATED("Use BigInt::serialize_to") void binary_encode(uint8_t buf[], size_t len) const;
 
       /**
        * Read integer value from a byte array with given size
        * @param buf byte array buffer containing the integer
        * @param length size of buf
        */
-      void binary_decode(const uint8_t buf[], size_t length);
+      BOTAN_DEPRECATED("Use BigInt::from_bytes") void binary_decode(const uint8_t buf[], size_t length) {
+         this->assign_from_bytes(std::span{buf, length});
+      }
 
       /**
        * Read integer value from a byte vector
        * @param buf the vector to load from
        */
-      void binary_decode(std::span<const uint8_t> buf) { binary_decode(buf.data(), buf.size()); }
+      BOTAN_DEPRECATED("Use BigInt::from_bytes") void binary_decode(std::span<const uint8_t> buf) {
+         this->assign_from_bytes(buf);
+      }
 
       /**
        * Place the value into out, zero-padding up to size words
        * Throw if *this cannot be represented in size words
        */
-      void encode_words(word out[], size_t size) const;
+      BOTAN_DEPRECATED("Deprecated no replacement") void encode_words(word out[], size_t size) const;
 
       /**
        * If predicate is true assign other to *this
        * Uses a masked operation to avoid side channels
        */
-      void ct_cond_assign(bool predicate, const BigInt& other);
+      BOTAN_DEPRECATED("Deprecated no replacement") void ct_cond_assign(bool predicate, const BigInt& other);
 
       /**
        * If predicate is true swap *this and other
        * Uses a masked operation to avoid side channels
        */
-      void ct_cond_swap(bool predicate, BigInt& other);
+      BOTAN_DEPRECATED("Deprecated no replacement") void ct_cond_swap(bool predicate, BigInt& other);
 
       /**
        * If predicate is true add value to *this
        */
-      void ct_cond_add(bool predicate, const BigInt& value);
+      BOTAN_DEPRECATED("Deprecated no replacement") void ct_cond_add(bool predicate, const BigInt& value);
 
       /**
        * Shift @p shift bits to the left, runtime is independent of
        * the value of @p shift.
        */
-      void ct_shift_left(size_t shift);
+      BOTAN_DEPRECATED("Deprecated no replacement") void ct_shift_left(size_t shift);
 
       /**
        * If predicate is true flip the sign of *this
        */
       void cond_flip_sign(bool predicate);
 
-#if defined(BOTAN_HAS_VALGRIND)
-      void const_time_poison() const;
-      void const_time_unpoison() const;
-#else
-      void const_time_poison() const {}
+      BOTAN_DEPRECATED("replaced by internal API") void const_time_poison() const { _const_time_poison(); }
 
-      void const_time_unpoison() const {}
-#endif
+      BOTAN_DEPRECATED("replaced by internal API") void const_time_unpoison() const { _const_time_unpoison(); }
 
       /**
        * @param rng a random number generator
@@ -747,10 +828,8 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param n the BigInt to use as integer source
        * @result secure_vector of bytes containing the bytes of the integer
        */
-      static std::vector<uint8_t> encode(const BigInt& n) {
-         std::vector<uint8_t> output(n.bytes());
-         n.binary_encode(output.data());
-         return output;
+      BOTAN_DEPRECATED("Use BigInt::serialize") static std::vector<uint8_t> encode(const BigInt& n) {
+         return n.serialize<std::vector<uint8_t>>(n.bytes());
       }
 
       /**
@@ -758,26 +837,35 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param n the BigInt to use as integer source
        * @result secure_vector of bytes containing the bytes of the integer
        */
-      static secure_vector<uint8_t> encode_locked(const BigInt& n) {
-         secure_vector<uint8_t> output(n.bytes());
-         n.binary_encode(output.data());
-         return output;
+      BOTAN_DEPRECATED("Use BigInt::serialize") static secure_vector<uint8_t> encode_locked(const BigInt& n) {
+         return n.serialize<secure_vector<uint8_t>>(n.bytes());
       }
 
+      /**
+       * Create a BigInt from an integer in a byte array
+       * @param bytes the binary value to load
+       * @result BigInt representing the integer in the byte array
+       */
+      static BigInt from_bytes(std::span<const uint8_t> bytes);
+
       /**
        * Create a BigInt from an integer in a byte array
        * @param buf the binary value to load
        * @param length size of buf
        * @result BigInt representing the integer in the byte array
        */
-      static BigInt decode(const uint8_t buf[], size_t length) { return BigInt(buf, length); }
+      BOTAN_DEPRECATED("Use BigInt::from_bytes") static BigInt decode(const uint8_t buf[], size_t length) {
+         return BigInt::from_bytes(std::span{buf, length});
+      }
 
       /**
        * Create a BigInt from an integer in a byte array
        * @param buf the binary value to load
        * @result BigInt representing the integer in the byte array
        */
-      static BigInt decode(std::span<const uint8_t> buf) { return BigInt(buf); }
+      BOTAN_DEPRECATED("Use BigInt::from_bytes") static BigInt decode(std::span<const uint8_t> buf) {
+         return BigInt::from_bytes(buf);
+      }
 
       /**
        * Create a BigInt from an integer in a byte array
@@ -786,6 +874,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param base number-base of the integer in buf
        * @result BigInt representing the integer in the byte array
        */
+      BOTAN_DEPRECATED("For decimal/hex use from_string")
       static BigInt decode(const uint8_t buf[], size_t length, Base base);
 
       /**
@@ -794,12 +883,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param base number-base of the integer in buf
        * @result BigInt representing the integer in the byte array
        */
-      static BigInt decode(std::span<const uint8_t> buf, Base base) {
-         if(base == Binary) {
-            return BigInt(buf);
-         }
-         return BigInt::decode(buf.data(), buf.size(), base);
-      }
+      BOTAN_DEPRECATED("For decimal/hex use from_string") static BigInt decode(std::span<const uint8_t> buf, Base base);
 
       /**
        * Encode a BigInt to a byte array according to IEEE 1363
@@ -807,10 +891,19 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param bytes the length of the resulting secure_vector<uint8_t>
        * @result a secure_vector<uint8_t> containing the encoded BigInt
        */
-      static secure_vector<uint8_t> encode_1363(const BigInt& n, size_t bytes);
-      static void encode_1363(std::span<uint8_t> out, const BigInt& n);
+      BOTAN_DEPRECATED("Use BigInt::serialize")
+      static secure_vector<uint8_t> encode_1363(const BigInt& n, size_t bytes) {
+         return n.serialize<secure_vector<uint8_t>>(bytes);
+      }
 
-      static void encode_1363(uint8_t out[], size_t bytes, const BigInt& n);
+      BOTAN_DEPRECATED("Use BigInt::serialize_to") static void encode_1363(std::span<uint8_t> out, const BigInt& n) {
+         n.serialize_to(out);
+      }
+
+      BOTAN_DEPRECATED("Use BigInt::serialize_to")
+      static void encode_1363(uint8_t out[], size_t bytes, const BigInt& n) {
+         n.serialize_to(std::span{out, bytes});
+      }
 
       /**
        * Encode two BigInt to a byte array according to IEEE 1363
@@ -819,9 +912,75 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
        * @param bytes the length of the encoding of each single BigInt
        * @result a secure_vector<uint8_t> containing the concatenation of the two encoded BigInt
        */
+      BOTAN_DEPRECATED("Deprecated no replacement")
       static secure_vector<uint8_t> encode_fixed_length_int_pair(const BigInt& n1, const BigInt& n2, size_t bytes);
 
+      /**
+       * Return a span over the register
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer.
+       *
+       * @result span over the internal register
+       */
+      std::span<const word> _as_span() const { return m_data.const_span(); }
+
+      /**
+       * Return a const pointer to the register
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer.
+       *
+       * @result a pointer to the start of the internal register
+       */
+      const word* _data() const { return m_data.const_data(); }
+
+      /**
+       * Read integer value from a byte vector (big endian)
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer. In applications use
+       * BigInt::from_bytes
+       *
+       * @param bytes the span of bytes to load
+       */
+      void _assign_from_bytes(std::span<const uint8_t> bytes) { assign_from_bytes(bytes); }
+
+      /**
+       * Create a BigInt from a word vector
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer.
+       */
+      static BigInt _from_words(secure_vector<word>&& words) {
+         BigInt bn;
+         bn.m_data.swap(words);
+         return bn;
+      }
+
+      /**
+       * Mark this BigInt as holding secret data
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer.
+       */
+      void _const_time_poison() const;
+
+      /**
+       * Mark this BigInt as no longer holding secret data
+       *
+       * @warning this is an implementation detail which is not for
+       * public use and not covered by SemVer.
+       */
+      void _const_time_unpoison() const;
+
    private:
+      /**
+       * Read integer value from a byte vector (big endian)
+       * @param bytes the span of bytes to load
+       */
+      void assign_from_bytes(std::span<const uint8_t> bytes);
+
       class Data {
          public:
             word* mutable_data() {
@@ -831,6 +990,8 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
 
             const word* const_data() const { return m_reg.data(); }
 
+            std::span<const word> const_span() const { return std::span{m_reg}; }
+
             secure_vector<word>& mutable_vector() {
                invalidate_sig_words();
                return m_reg;
@@ -861,36 +1022,9 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
                m_reg.assign(w, w + len);
             }
 
-            void set_to_zero() {
-               m_reg.resize(m_reg.capacity());
-               clear_mem(m_reg.data(), m_reg.size());
-               m_sig_words = 0;
-            }
+            void set_to_zero();
 
-            void set_size(size_t s) {
-               invalidate_sig_words();
-               clear_mem(m_reg.data(), m_reg.size());
-               m_reg.resize(s + (8 - (s % 8)));
-            }
-
-            void mask_bits(size_t n) {
-               if(n == 0) {
-                  return set_to_zero();
-               }
-
-               const size_t top_word = n / BOTAN_MP_WORD_BITS;
-
-               // if(top_word < sig_words()) ?
-               if(top_word < size()) {
-                  const word mask = (static_cast<word>(1) << (n % BOTAN_MP_WORD_BITS)) - 1;
-                  const size_t len = size() - (top_word + 1);
-                  if(len > 0) {
-                     clear_mem(&m_reg[top_word + 1], len);
-                  }
-                  m_reg[top_word] &= mask;
-                  invalidate_sig_words();
-               }
-            }
+            void mask_bits(size_t n);
 
             void grow_to(size_t n) const {
                if(n > size()) {
@@ -926,8 +1060,6 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
             size_t sig_words() const {
                if(m_sig_words == sig_words_npos) {
                   m_sig_words = calc_sig_words();
-               } else {
-                  BOTAN_DEBUG_ASSERT(m_sig_words == calc_sig_words());
                }
                return m_sig_words;
             }
@@ -949,7 +1081,7 @@ class BOTAN_PUBLIC_API(2, 0) BigInt final {
 * Arithmetic Operators
 */
 inline BigInt operator+(const BigInt& x, const BigInt& y) {
-   return BigInt::add2(x, y.data(), y.sig_words(), y.sign());
+   return BigInt::add2(x, y._data(), y.sig_words(), y.sign());
 }
 
 inline BigInt operator+(const BigInt& x, word y) {
@@ -961,26 +1093,26 @@ inline BigInt operator+(word x, const BigInt& y) {
 }
 
 inline BigInt operator-(const BigInt& x, const BigInt& y) {
-   return BigInt::add2(x, y.data(), y.sig_words(), y.reverse_sign());
+   return BigInt::add2(x, y._data(), y.sig_words(), y.reverse_sign());
 }
 
 inline BigInt operator-(const BigInt& x, word y) {
    return BigInt::add2(x, &y, 1, BigInt::Negative);
 }
 
-BigInt BOTAN_PUBLIC_API(2, 0) operator*(const BigInt& x, const BigInt& y);
-BigInt BOTAN_PUBLIC_API(2, 8) operator*(const BigInt& x, word y);
+BOTAN_PUBLIC_API(2, 0) BigInt operator*(const BigInt& x, const BigInt& y);
+BOTAN_PUBLIC_API(2, 8) BigInt operator*(const BigInt& x, word y);
 
 inline BigInt operator*(word x, const BigInt& y) {
    return y * x;
 }
 
-BigInt BOTAN_PUBLIC_API(2, 0) operator/(const BigInt& x, const BigInt& d);
-BigInt BOTAN_PUBLIC_API(2, 0) operator/(const BigInt& x, word m);
-BigInt BOTAN_PUBLIC_API(2, 0) operator%(const BigInt& x, const BigInt& m);
-word BOTAN_PUBLIC_API(2, 0) operator%(const BigInt& x, word m);
-BigInt BOTAN_PUBLIC_API(2, 0) operator<<(const BigInt& x, size_t n);
-BigInt BOTAN_PUBLIC_API(2, 0) operator>>(const BigInt& x, size_t n);
+BOTAN_PUBLIC_API(2, 0) BigInt operator/(const BigInt& x, const BigInt& d);
+BOTAN_PUBLIC_API(2, 0) BigInt operator/(const BigInt& x, word m);
+BOTAN_PUBLIC_API(2, 0) BigInt operator%(const BigInt& x, const BigInt& m);
+BOTAN_PUBLIC_API(2, 0) word operator%(const BigInt& x, word m);
+BOTAN_PUBLIC_API(2, 0) BigInt operator<<(const BigInt& x, size_t n);
+BOTAN_PUBLIC_API(2, 0) BigInt operator>>(const BigInt& x, size_t n);
 
 /*
  * Comparison Operators
@@ -1036,8 +1168,10 @@ inline bool operator>(const BigInt& a, word b) {
 /*
  * I/O Operators
  */
+BOTAN_DEPRECATED("Use BigInt::to_{hex,dec}_string")
 BOTAN_PUBLIC_API(2, 0) std::ostream& operator<<(std::ostream&, const BigInt&);
-BOTAN_PUBLIC_API(2, 0) std::istream& operator>>(std::istream&, BigInt&);
+
+BOTAN_DEPRECATED("Use BigInt::from_string") BOTAN_PUBLIC_API(2, 0) std::istream& operator>>(std::istream&, BigInt&);
 
 }  // namespace Botan
 
