@@ -10,6 +10,7 @@
 
 #include <botan/exceptn.h>
 #include <botan/secmem.h>
+#include <memory>
 #include <string>
 
 namespace Botan {
@@ -34,16 +35,26 @@ class BOTAN_PUBLIC_API(2, 0) Compression_Algorithm {
 
       /**
       * Begin compressing. Most compression algorithms offer a tunable
-      * time/compression tradeoff parameter generally represented by
-      * an integer in the range of 1 to 9.
+      * time/compression tradeoff parameter generally represented by an integer
+      * in the range of 1 to 9. Higher values typically imply better compression
+      * and more memory and/or CPU time consumed by the compression process.
       *
       * If 0 or a value out of range is provided, a compression algorithm
       * specific default is used.
+      *
+      * @param comp_level the desired level of compression (typically from 1 to 9)
       */
       virtual void start(size_t comp_level = 0) = 0;
 
       /**
       * Process some data.
+      *
+      * The leading @p offset bytes of @p buf are ignored and remain untouched;
+      * this can be useful for ignoring packet headers.  If @p flush is true,
+      * the compression state is flushed, allowing the decompressor to recover
+      * the entire message up to this point without having to see the rest of
+      * the compressed stream.
+      *
       * @param buf in/out parameter which will possibly be resized or swapped
       * @param offset an offset into blocks to begin processing
       * @param flush if true the compressor will be told to flush state
@@ -53,6 +64,10 @@ class BOTAN_PUBLIC_API(2, 0) Compression_Algorithm {
       /**
       * Finish compressing
       *
+      * The @p buf and @p offset parameters are treated as in update(). It is
+      * acceptable to call start() followed by finish() with the entire message,
+      * without any intervening call to update().
+      *
       * @param final_block in/out parameter
       * @param offset an offset into final_block to begin processing
       */
@@ -92,7 +107,10 @@ class BOTAN_PUBLIC_API(2, 0) Decompression_Algorithm {
 
       /**
       * Begin decompressing.
-      * Decompression does not support levels, as compression does.
+      *
+      * This initializes the decompression engine and must be done before
+      * calling update() or finish(). No level is provided here; the
+      * decompressor can accept input generated by any compression parameters.
       */
       virtual void start() = 0;
 
@@ -106,6 +124,12 @@ class BOTAN_PUBLIC_API(2, 0) Decompression_Algorithm {
       /**
       * Finish decompressing
       *
+      * Decompress the material in the in/out parameter @p buf. The leading
+      * @p offset bytes of @p buf are ignored and remain untouched; this can
+      * be useful for ignoring packet headers.
+      *
+      * This function may throw if the data seems to be invalid.
+      *
       * @param final_block in/out parameter
       * @param offset an offset into final_block to begin processing
       */
