@@ -12,13 +12,16 @@
 #define BOTAN_PKIX_TYPES_H_
 
 #include <botan/asn1_obj.h>
+
 #include <botan/pkix_enums.h>
-#include <vector>
-#include <string>
-#include <string_view>
 #include <iosfwd>
 #include <map>
+#include <memory>
 #include <set>
+#include <string>
+#include <string_view>
+#include <variant>
+#include <vector>
 
 namespace Botan {
 
@@ -26,30 +29,29 @@ class X509_Certificate;
 class Public_Key;
 
 BOTAN_DEPRECATED("Use Key_Constraints::to_string")
-inline std::string key_constraints_to_string(Key_Constraints c)
-   {
+
+inline std::string key_constraints_to_string(Key_Constraints c) {
    return c.to_string();
-   }
+}
 
 /**
 * Distinguished Name
 */
-class BOTAN_PUBLIC_API(2,0) X509_DN final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) X509_DN final : public ASN1_Object {
    public:
       X509_DN() = default;
 
-      explicit X509_DN(const std::multimap<OID, std::string>& args)
-         {
-         for(auto i : args)
+      explicit X509_DN(const std::multimap<OID, std::string>& args) {
+         for(const auto& i : args) {
             add_attribute(i.first, i.second);
          }
+      }
 
-      explicit X509_DN(const std::multimap<std::string, std::string>& args)
-         {
-         for(auto i : args)
+      explicit X509_DN(const std::multimap<std::string, std::string>& args) {
+         for(const auto& i : args) {
             add_attribute(i.first, i.second);
          }
+      }
 
       void encode_into(DER_Encoder&) const override;
       void decode_from(BER_Decoder&) override;
@@ -66,9 +68,11 @@ class BOTAN_PUBLIC_API(2,0) X509_DN final : public ASN1_Object
 
       bool empty() const { return m_rdn.empty(); }
 
+      size_t count() const { return m_rdn.size(); }
+
       std::string to_string() const;
 
-      const std::vector<std::pair<OID,ASN1_String>>& dn_info() const { return m_rdn; }
+      const std::vector<std::pair<OID, ASN1_String>>& dn_info() const { return m_rdn; }
 
       std::multimap<OID, std::string> get_attributes() const;
       std::multimap<std::string, std::string> contents() const;
@@ -79,10 +83,7 @@ class BOTAN_PUBLIC_API(2,0) X509_DN final : public ASN1_Object
 
       void add_attribute(std::string_view key, std::string_view val);
 
-      void add_attribute(const OID& oid, std::string_view val)
-         {
-         add_attribute(oid, ASN1_String(val));
-         }
+      void add_attribute(const OID& oid, std::string_view val) { add_attribute(oid, ASN1_String(val)); }
 
       void add_attribute(const OID& oid, const ASN1_String& val);
 
@@ -98,69 +99,126 @@ class BOTAN_PUBLIC_API(2,0) X509_DN final : public ASN1_Object
       static size_t lookup_ub(const OID& oid);
 
    private:
-      std::vector<std::pair<OID,ASN1_String>> m_rdn;
+      std::vector<std::pair<OID, ASN1_String>> m_rdn;
       std::vector<uint8_t> m_dn_bits;
-   };
+};
 
-bool BOTAN_PUBLIC_API(2,0) operator==(const X509_DN& dn1, const X509_DN& dn2);
-bool BOTAN_PUBLIC_API(2,0) operator!=(const X509_DN& dn1, const X509_DN& dn2);
+BOTAN_PUBLIC_API(2, 0) bool operator==(const X509_DN& dn1, const X509_DN& dn2);
+BOTAN_PUBLIC_API(2, 0) bool operator!=(const X509_DN& dn1, const X509_DN& dn2);
 
 /*
 The ordering here is arbitrary and may change from release to release.
 It is intended for allowing DNs as keys in std::map and similiar containers
 */
-bool BOTAN_PUBLIC_API(2,0) operator<(const X509_DN& dn1, const X509_DN& dn2);
+BOTAN_PUBLIC_API(2, 0) bool operator<(const X509_DN& dn1, const X509_DN& dn2);
 
-BOTAN_PUBLIC_API(2,0) std::ostream& operator<<(std::ostream& out, const X509_DN& dn);
-BOTAN_PUBLIC_API(2,0) std::istream& operator>>(std::istream& in, X509_DN& dn);
+BOTAN_PUBLIC_API(2, 0) std::ostream& operator<<(std::ostream& out, const X509_DN& dn);
+BOTAN_PUBLIC_API(2, 0) std::istream& operator>>(std::istream& in, X509_DN& dn);
 
 /**
 * Alternative Name
 */
-class BOTAN_PUBLIC_API(2,0) AlternativeName final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) AlternativeName final : public ASN1_Object {
    public:
       void encode_into(DER_Encoder&) const override;
       void decode_from(BER_Decoder&) override;
 
-      std::multimap<std::string, std::string> contents() const;
+      /// Create an empty name
+      AlternativeName() {}
 
-      bool has_field(std::string_view attr) const;
-      std::vector<std::string> get_attribute(std::string_view attr) const;
+      /// Add a URI to this AlternativeName
+      void add_uri(std::string_view uri);
 
-      std::string get_first_attribute(std::string_view attr) const;
+      /// Add a URI to this AlternativeName
+      void add_email(std::string_view addr);
 
-      void add_attribute(std::string_view type, std::string_view value);
-      void add_othername(const OID& oid, std::string_view value, ASN1_Type type);
+      /// Add a DNS name to this AlternativeName
+      void add_dns(std::string_view dns);
 
-      const std::multimap<std::string, std::string, std::less<>>& get_attributes() const
-         {
-         return m_alt_info;
-         }
+      /// Add an "OtherName" identified by object identifier to this AlternativeName
+      void add_other_name(const OID& oid, const ASN1_String& value);
 
-      const std::multimap<OID, ASN1_String>& get_othernames() const
-         {
+      /// Add a directory name to this AlternativeName
+      void add_dn(const X509_DN& dn);
+
+      /// Add an IP address to this alternative name
+      void add_ipv4_address(uint32_t ipv4);
+
+      /// Return the set of URIs included in this alternative name
+      const std::set<std::string>& uris() const { return m_uri; }
+
+      /// Return the set of email addresses included in this alternative name
+      const std::set<std::string>& email() const { return m_email; }
+
+      /// Return the set of DNS names included in this alternative name
+      const std::set<std::string>& dns() const { return m_dns; }
+
+      /// Return the set of IPv4 addresses included in this alternative name
+      const std::set<uint32_t>& ipv4_address() const { return m_ipv4_addr; }
+
+      /// Return the set of "other names" included in this alternative name
+      BOTAN_DEPRECATED("Support for other names is deprecated")
+      const std::set<std::pair<OID, ASN1_String>>& other_names() const {
          return m_othernames;
-         }
+      }
 
-      X509_DN dn() const;
+      /// Return the set of directory names included in this alternative name
+      const std::set<X509_DN>& directory_names() const { return m_dn_names; }
 
+      /// Return the total number of names in this AlternativeName
+      ///
+      /// This only counts names which were parsed, ignoring names which
+      /// were of some unknown type
+      size_t count() const;
+
+      /// Return true if this has any names set
       bool has_items() const;
 
-      AlternativeName(std::string_view email_addr = "",
+      // Old, now deprecated interface follows:
+      BOTAN_DEPRECATED("Use AlternativeName::{uris, email, dns, othernames, directory_names}")
+      std::multimap<std::string, std::string> contents() const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::{uris, email, dns, othernames, directory_names}.empty()")
+      bool has_field(std::string_view attr) const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::{uris, email, dns, othernames, directory_names}")
+      std::vector<std::string> get_attribute(std::string_view attr) const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::{uris, email, dns, othernames, directory_names}")
+      std::multimap<std::string, std::string, std::less<>> get_attributes() const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::{uris, email, dns, othernames, directory_names}")
+      std::string get_first_attribute(std::string_view attr) const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::add_{uri, dns, email, ...}")
+      void add_attribute(std::string_view type, std::string_view value);
+
+      BOTAN_DEPRECATED("Use AlternativeName::add_other_name")
+      void add_othername(const OID& oid, std::string_view value, ASN1_Type type);
+
+      BOTAN_DEPRECATED("Use AlternativeName::othernames") std::multimap<OID, ASN1_String> get_othernames() const;
+
+      BOTAN_DEPRECATED("Use AlternativeName::directory_names") X509_DN dn() const;
+
+      BOTAN_DEPRECATED("Use plain constructor plus add_{uri,dns,email,ipv4_address}")
+      AlternativeName(std::string_view email_addr,
                       std::string_view uri = "",
                       std::string_view dns = "",
                       std::string_view ip_address = "");
+
    private:
-      std::multimap<std::string, std::string, std::less<>> m_alt_info;
-      std::multimap<OID, ASN1_String> m_othernames;
-   };
+      std::set<std::string> m_dns;
+      std::set<std::string> m_uri;
+      std::set<std::string> m_email;
+      std::set<uint32_t> m_ipv4_addr;
+      std::set<X509_DN> m_dn_names;
+      std::set<std::pair<OID, ASN1_String>> m_othernames;
+};
 
 /**
 * Attribute
 */
-class BOTAN_PUBLIC_API(2,0) Attribute final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) Attribute final : public ASN1_Object {
    public:
       void encode_into(DER_Encoder& to) const override;
       void decode_from(BER_Decoder& from) override;
@@ -170,15 +228,17 @@ class BOTAN_PUBLIC_API(2,0) Attribute final : public ASN1_Object
       Attribute(std::string_view oid_str, const std::vector<uint8_t>& params);
 
       const OID& oid() const { return m_oid; }
+
       const std::vector<uint8_t>& parameters() const { return m_parameters; }
 
       const OID& object_identifier() const { return m_oid; }
+
       const std::vector<uint8_t>& get_parameters() const { return m_parameters; }
 
    private:
       OID m_oid;
       std::vector<uint8_t> m_parameters;
-   };
+};
 
 /**
 * @brief X.509 GeneralName Type
@@ -186,93 +246,96 @@ class BOTAN_PUBLIC_API(2,0) Attribute final : public ASN1_Object
 * Handles parsing GeneralName types in their BER and canonical string
 * encoding. Allows matching GeneralNames against each other using
 * the rules laid out in the RFC 5280, sec. 4.2.1.10 (Name Contraints).
+*
+* This entire class is deprecated and will be removed in a future
+* major release
 */
-class BOTAN_PUBLIC_API(2,0) GeneralName final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) GeneralName final : public ASN1_Object {
    public:
-      enum MatchResult : int
-            {
-            All,
-            Some,
-            None,
-            NotFound,
-            UnknownType,
-            };
+      enum MatchResult : int {
+         All,
+         Some,
+         None,
+         NotFound,
+         UnknownType,
+      };
 
-      /**
-      * Creates an empty GeneralName.
-      */
-      GeneralName() = default;
+      enum class NameType : uint8_t {
+         Unknown = 0,
+         RFC822 = 1,
+         DNS = 2,
+         URI = 3,
+         DN = 4,
+         IPv4 = 5,
+         Other = 6,
+      };
 
-      /**
-      * Creates a new GeneralName for its string format.
-      * @param str type and name, colon-separated, e.g., "DNS:google.com"
-      */
-      GeneralName(const std::string& str);
+      BOTAN_DEPRECATED("Deprecated use NameConstraints") GeneralName() = default;
 
+      // Encoding is not implemented
       void encode_into(DER_Encoder&) const override;
 
       void decode_from(BER_Decoder&) override;
 
+      /**
+      * @return Type of the name expressed in this restriction
+      */
+      NameType type_code() const { return m_type; }
+
       /**
       * @return Type of the name. Can be DN, DNS, IP, RFC822 or URI.
       */
-      const std::string& type() const { return m_type; }
+      BOTAN_DEPRECATED("Deprecated use type_code") std::string type() const;
 
       /**
       * @return The name as string. Format depends on type.
       */
-      const std::string& name() const { return m_name; }
+      BOTAN_DEPRECATED("Deprecated no replacement") std::string name() const;
 
       /**
       * Checks whether a given certificate (partially) matches this name.
       * @param cert certificate to be matched
       * @return the match result
       */
-      MatchResult matches(const X509_Certificate& cert) const;
+      BOTAN_DEPRECATED("Deprecated use NameConstraints type") MatchResult matches(const X509_Certificate& cert) const;
+
+      bool matches_dns(const std::string& dns_name) const;
+      bool matches_ipv4(uint32_t ip) const;
+      bool matches_dn(const X509_DN& dn) const;
 
    private:
-      std::string m_type;
-      std::string m_name;
+      static constexpr size_t RFC822_IDX = 0;
+      static constexpr size_t DNS_IDX = 1;
+      static constexpr size_t URI_IDX = 2;
+      static constexpr size_t DN_IDX = 3;
+      static constexpr size_t IPV4_IDX = 4;
 
-      bool matches_dns(const std::string&) const;
-      bool matches_dn(const std::string&) const;
-      bool matches_ip(const std::string&) const;
-   };
+      NameType m_type;
+      std::variant<std::string, std::string, std::string, X509_DN, std::pair<uint32_t, uint32_t>> m_name;
 
-std::ostream& operator<<(std::ostream& os, const GeneralName& gn);
+      static bool matches_dns(std::string_view name, std::string_view constraint);
+
+      static bool matches_dn(const X509_DN& name, const X509_DN& constraint);
+};
+
+BOTAN_DEPRECATED("Deprecated no replacement") std::ostream& operator<<(std::ostream& os, const GeneralName& gn);
 
 /**
 * @brief A single Name Constraint
 *
 * The Name Constraint extension adds a minimum and maximum path
 * length to a GeneralName to form a constraint. The length limits
-* are currently unused.
+* are not used in PKIX.
+*
+* This entire class is deprecated and will be removed in a future
+* major release
 */
-class BOTAN_PUBLIC_API(2,0) GeneralSubtree final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) GeneralSubtree final : public ASN1_Object {
    public:
       /**
       * Creates an empty name constraint.
       */
-      GeneralSubtree() : m_base(), m_minimum(0), m_maximum(std::numeric_limits<std::size_t>::max())
-      {}
-
-      /***
-      * Creates a new name constraint.
-      * @param base name
-      * @param min minimum path length
-      * @param max maximum path length
-      */
-      GeneralSubtree(const GeneralName& base, size_t min, size_t max)
-      : m_base(base), m_minimum(min), m_maximum(max)
-      {}
-
-      /**
-      * Creates a new name constraint for its string format.
-      * @param str name constraint
-      */
-      GeneralSubtree(const std::string& str);
+      BOTAN_DEPRECATED("Deprecated use NameConstraints") GeneralSubtree();
 
       void encode_into(DER_Encoder&) const override;
 
@@ -283,31 +346,18 @@ class BOTAN_PUBLIC_API(2,0) GeneralSubtree final : public ASN1_Object
       */
       const GeneralName& base() const { return m_base; }
 
-      /**
-      * @return minimum path length
-      */
-      size_t minimum() const { return m_minimum; }
-
-      /**
-      * @return maximum path length
-      */
-      size_t maximum() const { return m_maximum; }
-
    private:
       GeneralName m_base;
-      size_t m_minimum;
-      size_t m_maximum;
-   };
+};
 
-std::ostream& operator<<(std::ostream& os, const GeneralSubtree& gs);
+BOTAN_DEPRECATED("Deprecated no replacement") std::ostream& operator<<(std::ostream& os, const GeneralSubtree& gs);
 
 /**
 * @brief Name Constraints
 *
 * Wraps the Name Constraints associated with a certificate.
 */
-class BOTAN_PUBLIC_API(2,0) NameConstraints final
-   {
+class BOTAN_PUBLIC_API(2, 0) NameConstraints final {
    public:
       /**
       * Creates an empty name NameConstraints.
@@ -320,30 +370,44 @@ class BOTAN_PUBLIC_API(2,0) NameConstraints final
       * @param excluded_subtrees names for which the certificate is not permitted
       */
       NameConstraints(std::vector<GeneralSubtree>&& permitted_subtrees,
-                    std::vector<GeneralSubtree>&& excluded_subtrees)
-      : m_permitted_subtrees(permitted_subtrees), m_excluded_subtrees(excluded_subtrees)
-      {}
+                      std::vector<GeneralSubtree>&& excluded_subtrees);
 
       /**
       * @return permitted names
       */
-      const std::vector<GeneralSubtree>& permitted() const { return m_permitted_subtrees; }
+      BOTAN_DEPRECATED("Deprecated no replacement") const std::vector<GeneralSubtree>& permitted() const {
+         return m_permitted_subtrees;
+      }
 
       /**
       * @return excluded names
       */
-      const std::vector<GeneralSubtree>& excluded() const { return m_excluded_subtrees; }
+      BOTAN_DEPRECATED("Deprecated no replacement") const std::vector<GeneralSubtree>& excluded() const {
+         return m_excluded_subtrees;
+      }
+
+      /**
+      * Return true if all of the names in the certificate are permitted
+      */
+      bool is_permitted(const X509_Certificate& cert, bool reject_unknown) const;
+
+      /**
+      * Return true if any of the names in the certificate are excluded
+      */
+      bool is_excluded(const X509_Certificate& cert, bool reject_unknown) const;
 
    private:
       std::vector<GeneralSubtree> m_permitted_subtrees;
       std::vector<GeneralSubtree> m_excluded_subtrees;
-   };
+
+      std::set<GeneralName::NameType> m_permitted_name_types;
+      std::set<GeneralName::NameType> m_excluded_name_types;
+};
 
 /**
 * X.509 Certificate Extension
 */
-class BOTAN_PUBLIC_API(2,0) Certificate_Extension
-   {
+class BOTAN_PUBLIC_API(2, 0) Certificate_Extension {
    public:
       /**
       * @return OID representing this extension
@@ -380,24 +444,27 @@ class BOTAN_PUBLIC_API(2,0) Certificate_Extension
       * @param cert_path Certificate path which is currently validated
       * @param pos Position of subject certificate in cert_path
       */
-      virtual void validate(const X509_Certificate& subject, const X509_Certificate& issuer,
-            const std::vector<X509_Certificate>& cert_path,
-            std::vector<std::set<Certificate_Status_Code>>& cert_status,
-            size_t pos);
+      virtual void validate(const X509_Certificate& subject,
+                            const X509_Certificate& issuer,
+                            const std::vector<X509_Certificate>& cert_path,
+                            std::vector<std::set<Certificate_Status_Code>>& cert_status,
+                            size_t pos);
 
       virtual ~Certificate_Extension() = default;
+
    protected:
       friend class Extensions;
+
       virtual bool should_encode() const { return true; }
+
       virtual std::vector<uint8_t> encode_inner() const = 0;
       virtual void decode_inner(const std::vector<uint8_t>&) = 0;
-   };
+};
 
 /**
 * X.509 Certificate Extension List
 */
-class BOTAN_PUBLIC_API(2,0) Extensions final : public ASN1_Object
-   {
+class BOTAN_PUBLIC_API(2, 0) Extensions final : public ASN1_Object {
    public:
       /**
       * Look up an object in the extensions, based on OID Returns
@@ -408,37 +475,27 @@ class BOTAN_PUBLIC_API(2,0) Extensions final : public ASN1_Object
       */
       const Certificate_Extension* get_extension_object(const OID& oid) const;
 
-      template<typename T>
-      const T* get_extension_object_as(const OID& oid = T::static_oid()) const
-         {
-         if(const Certificate_Extension* extn = get_extension_object(oid))
-            {
+      template <typename T>
+      const T* get_extension_object_as(const OID& oid = T::static_oid()) const {
+         if(const Certificate_Extension* extn = get_extension_object(oid)) {
             // Unknown_Extension oid_name is empty
-            if(extn->oid_name().empty())
-               {
+            if(extn->oid_name().empty()) {
                return nullptr;
-               }
-            else if(const T* extn_as_T = dynamic_cast<const T*>(extn))
-               {
+            } else if(const T* extn_as_T = dynamic_cast<const T*>(extn)) {
                return extn_as_T;
-               }
-            else
-               {
+            } else {
                throw Decoding_Error("Exception::get_extension_object_as dynamic_cast failed");
-               }
             }
+         }
 
          return nullptr;
-         }
+      }
 
       /**
       * Return the set of extensions in the order they appeared in the certificate
       * (or as they were added, if constructed)
       */
-      const std::vector<OID>& get_extension_oids() const
-         {
-         return m_extension_oids;
-         }
+      const std::vector<OID>& get_extension_oids() const { return m_extension_oids; }
 
       /**
       * Return true if an extension was set
@@ -509,23 +566,20 @@ class BOTAN_PUBLIC_API(2,0) Extensions final : public ASN1_Object
       *
       * @return Pointer to new extension with oid, nullptr if not found.
       */
-      template<typename T>
-      std::unique_ptr<T> get_raw(const OID& oid) const
-         {
+      template <typename T>
+      std::unique_ptr<T> get_raw(const OID& oid) const {
          auto extn_info = m_extension_info.find(oid);
 
-         if(extn_info != m_extension_info.end())
-            {
+         if(extn_info != m_extension_info.end()) {
             // Unknown_Extension oid_name is empty
-            if(extn_info->second.obj().oid_name() == "")
-               {
+            if(extn_info->second.obj().oid_name().empty()) {
                auto ext = std::make_unique<T>();
                ext->decode_inner(extn_info->second.bits());
                return ext;
-               }
             }
-         return nullptr;
          }
+         return nullptr;
+      }
 
       /**
       * Returns a copy of the list of extensions together with the corresponding
@@ -542,7 +596,7 @@ class BOTAN_PUBLIC_API(2,0) Extensions final : public ASN1_Object
       */
       std::map<OID, std::pair<std::vector<uint8_t>, bool>> extensions_raw() const;
 
-      Extensions() {}
+      Extensions() = default;
 
       Extensions(const Extensions&) = default;
       Extensions& operator=(const Extensions&) = default;
@@ -551,49 +605,36 @@ class BOTAN_PUBLIC_API(2,0) Extensions final : public ASN1_Object
       Extensions& operator=(Extensions&&) = default;
 
    private:
-      static std::unique_ptr<Certificate_Extension>
-         create_extn_obj(const OID& oid,
-                         bool critical,
-                         const std::vector<uint8_t>& body);
+      static std::unique_ptr<Certificate_Extension> create_extn_obj(const OID& oid,
+                                                                    bool critical,
+                                                                    const std::vector<uint8_t>& body);
 
-      class Extensions_Info
-         {
+      class BOTAN_UNSTABLE_API Extensions_Info {
          public:
-            Extensions_Info(bool critical,
-                            std::unique_ptr<Certificate_Extension> ext) :
-               m_obj(std::move(ext)),
-               m_bits(m_obj->encode_inner()),
-               m_critical(critical)
-               {
-               }
+            Extensions_Info(bool critical, std::unique_ptr<Certificate_Extension> ext) :
+                  m_obj(std::move(ext)), m_bits(m_obj->encode_inner()), m_critical(critical) {}
 
             Extensions_Info(bool critical,
                             const std::vector<uint8_t>& encoding,
                             std::unique_ptr<Certificate_Extension> ext) :
-               m_obj(std::move(ext)),
-               m_bits(encoding),
-               m_critical(critical)
-               {
-               }
+                  m_obj(std::move(ext)), m_bits(encoding), m_critical(critical) {}
 
             bool is_critical() const { return m_critical; }
+
             const std::vector<uint8_t>& bits() const { return m_bits; }
-            const Certificate_Extension& obj() const
-               {
-               BOTAN_ASSERT_NONNULL(m_obj.get());
-               return *m_obj.get();
-               }
+
+            const Certificate_Extension& obj() const;
 
          private:
             std::shared_ptr<Certificate_Extension> m_obj;
             std::vector<uint8_t> m_bits;
             bool m_critical = false;
-         };
+      };
 
       std::vector<OID> m_extension_oids;
       std::map<OID, Extensions_Info> m_extension_info;
-   };
+};
 
-}
+}  // namespace Botan
 
 #endif
