@@ -10,6 +10,7 @@
 #include <botan/secmem.h>
 #include <memory>
 #include <set>
+#include <span>
 #include <string>
 
 namespace Botan {
@@ -25,12 +26,12 @@ class MessageAuthenticationCode;
 class BOTAN_PUBLIC_API(2, 4) PSK_Database {
    public:
       /**
-      * Return the set of names for which get() will return a value.
+      * @returns the set of names for which get() will return a value.
       */
       virtual std::set<std::string> list_names() const = 0;
 
       /**
-      * Return the value associated with the specified @param name or otherwise
+      * @returns the value associated with the specified @p name or otherwise
       * throw an exception.
       */
       virtual secure_vector<uint8_t> get(std::string_view name) const = 0;
@@ -42,13 +43,13 @@ class BOTAN_PUBLIC_API(2, 4) PSK_Database {
       virtual void set(std::string_view name, const uint8_t psk[], size_t psk_len) = 0;
 
       /**
-      * Remove a PSK from the database
+      * Remove the PSK with the given @p name from the database
       */
       virtual void remove(std::string_view name) = 0;
 
       /**
-      * Returns if the values in the PSK database are encrypted. If
-      * false, saved values are being stored in plaintext.
+      * @returns true if the values in the PSK database are encrypted. If false,
+      *          saved values are being stored in plaintext.
       */
       virtual bool is_encrypted() const = 0;
 
@@ -57,34 +58,59 @@ class BOTAN_PUBLIC_API(2, 4) PSK_Database {
       */
       std::string get_str(std::string_view name) const;
 
+      /**
+      * Like set() but accepts the PSK as a string (eg for a password).
+      */
       void set_str(std::string_view name, std::string_view psk);
 
-      template <typename Alloc>
-      void set_vec(std::string_view name, const std::vector<uint8_t, Alloc>& psk) {
-         set(name, psk.data(), psk.size());
-      }
+      /**
+      * Like set() but accepting an arbitrary contiguous byte array.
+      */
+      void set_vec(std::string_view name, std::span<const uint8_t> psk) { set(name, psk.data(), psk.size()); }
 
       virtual ~PSK_Database() = default;
 };
 
 /**
 * A mixin for an encrypted PSK database.
-* Both keys and values are encrypted with NIST AES-256 key wrapping.
-* Values are padded to obscure their length before encryption, allowing
-* it to be used as a password vault.
 *
-* Subclasses must implement the virtual calls to handle storing and
-* getting raw (base64 encoded) values.
+* Both names and values are encrypted using NIST key wrapping (see NIST
+* SP800-38F) with AES-256. First the master key is used with HMAC(SHA-256) to
+* derive two 256-bit keys, one for encrypting all names and the other to key an
+* instance of HMAC(SHA-256). Values are each encrypted under an individual key
+* created by hashing the encrypted name with HMAC. This associates the encrypted
+* key with the name, and prevents an attacker with write access to the data
+* store from taking an encrypted key associated with one entity and copying it
+* to another entity.
+*
+* Names and PSKs are both padded to the next multiple of 8 bytes, providing some
+* obfuscation of the length.
+*
+* Subclasses must implement the virtual calls to handle storing and getting raw
+* (base64 encoded) values.
 */
 class BOTAN_PUBLIC_API(2, 4) Encrypted_PSK_Database : public PSK_Database {
    public:
       /**
+      * Initializes or opens a PSK database. The @p master_key is used to secure
+      * the contents. It may be of any length. If encrypting PSKs under a
+      * passphrase, use a suitable key derivation scheme (such as Argon2id) to
+      * derive the secret key. If the master key is lost, all PSKs stored are
+      * unrecoverable.
+      *
+      * One artifact of the names being encrypted is that is is possible to use
+      * multiple different master keys with the same underlying storage. Each
+      * master key will be responsible for a subset of the keys. An attacker who
+      * knows one of the keys will be able to tell there are other values
+      * encrypted under another key, but will not be able to tell how many other
+      * master keys are in use.
+      *
       * @param master_key specifies the master key used to encrypt all
       * keys and value. It can be of any length, but should be at least 256 bits.
       *
       * Subkeys for the cryptographic algorithms used are derived from this
       * master key. No key stretching is performed; if encrypting a PSK database
-      * using a password, it is recommended to use PBKDF2 to derive the database
+      * using a password, it is recommended to use Argon2id to derive the database
       * master key.
       */
       Encrypted_PSK_Database(const secure_vector<uint8_t>& master_key);
@@ -103,23 +129,25 @@ class BOTAN_PUBLIC_API(2, 4) Encrypted_PSK_Database : public PSK_Database {
 
    protected:
       /**
-      * Save a encrypted (name.value) pair to the database. Both will be base64 encoded strings.
+      * Save a encrypted (name/value) pair to the database. Both will be base64
+      * encoded strings.
       */
       virtual void kv_set(std::string_view index, std::string_view value) = 0;
 
       /**
-      * Get a value previously saved with set_raw_value. Should return an empty
-      * string if index is not found.
+      * Get a value previously saved with kv_set(). Should return an empty
+      * string if @p index is not found.
       */
       virtual std::string kv_get(std::string_view index) const = 0;
 
       /**
-      * Remove an index
+      * Remove an @p index
       */
       virtual void kv_del(std::string_view index) = 0;
 
       /**
-      * Return all indexes in the table.
+      * Return all indexes in the table (ie values for which ``kv_get`` will
+      * return a non-empty string)
       */
       virtual std::set<std::string> kv_get_all() const = 0;
 
@@ -133,6 +161,10 @@ class SQL_Database;
 
 class BOTAN_PUBLIC_API(2, 4) Encrypted_PSK_Database_SQL : public Encrypted_PSK_Database {
    public:
+      /**
+      * Creates or uses the named table in @p db. The SQL schema of the table is
+      * `(psk_name TEXT PRIMARY KEY, psk_value TEXT)`.
+      */
       Encrypted_PSK_Database_SQL(const secure_vector<uint8_t>& master_key,
                                  std::shared_ptr<SQL_Database> db,
                                  std::string_view table_name);
