@@ -9,11 +9,12 @@
 #ifndef BOTAN_STRONG_TYPE_H_
 #define BOTAN_STRONG_TYPE_H_
 
-#include <ostream>
-#include <span>
-
 #include <botan/concepts.h>
 
+#include <iosfwd>
+#include <span>
+#include <string>
+
 namespace Botan {
 
 /**
@@ -47,9 +48,13 @@ class Strong_Base {
 
       constexpr explicit Strong_Base(T v) : m_value(std::move(v)) {}
 
-      T& get() { return m_value; }
+      constexpr T& get() & { return m_value; }
 
-      const T& get() const { return m_value; }
+      constexpr const T& get() const& { return m_value; }
+
+      constexpr T&& get() && { return std::move(m_value); }
+
+      constexpr const T&& get() const&& { return std::move(m_value); }
 };
 
 template <typename T>
@@ -65,35 +70,22 @@ class Strong_Adapter<T> : public Strong_Base<T> {
 };
 
 template <concepts::container T>
-class Strong_Adapter<T> : public Strong_Base<T> {
+class Container_Strong_Adapter_Base : public Strong_Base<T> {
    public:
       using value_type = typename T::value_type;
       using size_type = typename T::size_type;
       using iterator = typename T::iterator;
       using const_iterator = typename T::const_iterator;
-      using pointer = typename T::pointer;
-      using const_pointer = typename T::const_pointer;
 
    public:
       using Strong_Base<T>::Strong_Base;
 
-      explicit Strong_Adapter(std::span<const value_type> span)
-         requires(concepts::contiguous_container<T>)
-            : Strong_Adapter(T(span.begin(), span.end())) {}
-
-      explicit Strong_Adapter(size_t size)
+      explicit Container_Strong_Adapter_Base(size_t size)
          requires(concepts::resizable_container<T>)
-            : Strong_Adapter(T(size)) {}
+            : Container_Strong_Adapter_Base(T(size)) {}
 
       template <typename InputIt>
-      Strong_Adapter(InputIt begin, InputIt end) : Strong_Adapter(T(begin, end)) {}
-
-      // Disambiguates the usage of string literals, otherwise:
-      // Strong_Adapter(std::span<>) and Strong_Adapter(const char*)
-      // would be ambiguous.
-      explicit Strong_Adapter(const char* str)
-         requires(std::same_as<T, std::string>)
-            : Strong_Adapter(std::string(str)) {}
+      Container_Strong_Adapter_Base(InputIt begin, InputIt end) : Container_Strong_Adapter_Base(T(begin, end)) {}
 
    public:
       decltype(auto) begin() noexcept(noexcept(this->get().begin())) { return this->get().begin(); }
@@ -114,18 +106,6 @@ class Strong_Adapter<T> : public Strong_Base<T> {
 
       size_type size() const noexcept(noexcept(this->get().size())) { return this->get().size(); }
 
-      decltype(auto) data() noexcept(noexcept(this->get().data()))
-         requires(concepts::contiguous_container<T>)
-      {
-         return this->get().data();
-      }
-
-      decltype(auto) data() const noexcept(noexcept(this->get().data()))
-         requires(concepts::contiguous_container<T>)
-      {
-         return this->get().data();
-      }
-
       bool empty() const noexcept(noexcept(this->get().empty()))
          requires(concepts::has_empty<T>)
       {
@@ -144,11 +124,60 @@ class Strong_Adapter<T> : public Strong_Base<T> {
          this->get().reserve(size);
       }
 
-      decltype(auto) operator[](size_type i) const noexcept(noexcept(this->get().operator[](i))) {
-         return this->get()[i];
+      template <typename U>
+      decltype(auto) operator[](U&& i) const noexcept(noexcept(this->get().operator[](i))) {
+         return this->get()[std::forward<U>(i)];
       }
 
-      decltype(auto) operator[](size_type i) noexcept(noexcept(this->get().operator[](i))) { return this->get()[i]; }
+      template <typename U>
+      decltype(auto) operator[](U&& i) noexcept(noexcept(this->get().operator[](i))) {
+         return this->get()[std::forward<U>(i)];
+      }
+
+      template <typename U>
+      decltype(auto) at(U&& i) const noexcept(noexcept(this->get().at(i)))
+         requires(concepts::has_bounds_checked_accessors<T>)
+      {
+         return this->get().at(std::forward<U>(i));
+      }
+
+      template <typename U>
+      decltype(auto) at(U&& i) noexcept(noexcept(this->get().at(i)))
+         requires(concepts::has_bounds_checked_accessors<T>)
+      {
+         return this->get().at(std::forward<U>(i));
+      }
+};
+
+template <concepts::container T>
+class Strong_Adapter<T> : public Container_Strong_Adapter_Base<T> {
+   public:
+      using Container_Strong_Adapter_Base<T>::Container_Strong_Adapter_Base;
+};
+
+template <concepts::contiguous_container T>
+class Strong_Adapter<T> : public Container_Strong_Adapter_Base<T> {
+   public:
+      using pointer = typename T::pointer;
+      using const_pointer = typename T::const_pointer;
+
+   public:
+      using Container_Strong_Adapter_Base<T>::Container_Strong_Adapter_Base;
+
+      explicit Strong_Adapter(std::span<const typename Container_Strong_Adapter_Base<T>::value_type> span) :
+            Strong_Adapter(T(span.begin(), span.end())) {}
+
+      // Disambiguates the usage of string literals, otherwise:
+      // Strong_Adapter(std::span<>) and Strong_Adapter(const char*)
+      // would be ambiguous.
+      explicit Strong_Adapter(const char* str)
+         requires(std::same_as<T, std::string>)
+            : Strong_Adapter(std::string(str)) {}
+
+   public:
+      decltype(auto) data() noexcept(noexcept(this->get().data())) { return this->get().data(); }
+
+      decltype(auto) data() const noexcept(noexcept(this->get().data())) { return this->get().data(); }
 };
 
 }  // namespace detail
@@ -179,18 +208,84 @@ class Strong : public detail::Strong_Adapter<T> {
 };
 
 /**
- * Opportunistically unpacks a strong type to its underlying type. If the
- * provided type is not a strong type, it is returned as is.
+ * @brief Generically unwraps a strong type to its underlying type.
+ *
+ * If the provided type is not a strong type, it is returned as is.
+ *
+ * @note This is meant as a helper for generic code that needs to deal with both
+ *       wrapped strong types and bare objects. Use the ordinary `get()` method
+ *       if you know that you are dealing with a strong type.
+ *
+ * @param t  value to be unwrapped
+ * @return   the unwrapped value
  */
 template <typename T>
-constexpr decltype(auto) unpack(T& t) {
-   if constexpr(concepts::strong_type<std::remove_cvref_t<T>>) {
-      return t.get();
+[[nodiscard]] constexpr decltype(auto) unwrap_strong_type(T&& t) {
+   if constexpr(!concepts::strong_type<std::remove_cvref_t<T>>) {
+      // If the parameter type isn't a strong type, return it as is.
+      return std::forward<T>(t);
    } else {
-      return t;
+      // Unwrap the strong type and return the underlying value.
+      return std::forward<T>(t).get();
    }
 }
 
+/**
+ * @brief Wraps a value into a caller-defined (strong) type.
+ *
+ * If the provided object @p t is already of type @p T, it is returned as is.
+ *
+ * @note This is meant as a helper for generic code that needs to deal with both
+ *       wrapped strong types and bare objects. Use the ordinary constructor if
+ *       you know that you are dealing with a bare value type.
+ *
+ * @param t  value to be wrapped
+ * @return   the wrapped value
+ */
+template <typename T, typename ParamT>
+   requires std::constructible_from<T, ParamT> ||
+            (concepts::strong_type<T> && std::constructible_from<typename T::wrapped_type, ParamT>)
+[[nodiscard]] constexpr decltype(auto) wrap_strong_type(ParamT&& t) {
+   if constexpr(std::same_as<std::remove_cvref_t<ParamT>, T>) {
+      // Noop, if the parameter type already is the desired return type.
+      return std::forward<ParamT>(t);
+   } else if constexpr(std::constructible_from<T, ParamT>) {
+      // Implicit conversion from the parameter type to the return type.
+      return T{std::forward<ParamT>(t)};
+   } else {
+      // Explicitly calling the wrapped type's constructor to support
+      // implicit conversions on types that mark their constructors as explicit.
+      static_assert(concepts::strong_type<T> && std::constructible_from<typename T::wrapped_type, ParamT>);
+      return T{typename T::wrapped_type{std::forward<ParamT>(t)}};
+   }
+}
+
+namespace detail {
+
+template <typename T>
+struct wrapped_type_helper {
+      using type = T;
+};
+
+template <concepts::strong_type T>
+struct wrapped_type_helper<T> {
+      using type = typename T::wrapped_type;
+};
+
+}  // namespace detail
+
+/**
+ * @brief Extracts the wrapped type from a strong type.
+ *
+ * If the provided type is not a strong type, it is returned as is.
+ *
+ * @note This is meant as a helper for generic code that needs to deal with both
+ *       wrapped strong types and bare objects. Use the ordinary `::wrapped_type`
+ *       declaration if you know that you are dealing with a strong type.
+ */
+template <typename T>
+using strong_type_wrapped_type = typename detail::wrapped_type_helper<std::remove_cvref_t<T>>::type;
+
 template <typename T, typename... Tags>
    requires(concepts::streamable<T>)
 decltype(auto) operator<<(std::ostream& os, const Strong<T, Tags...>& v) {
